;;**********************************************************************;;
;;									;;
;;		E m a c s   K e y   B i n d i n g s 			;;
;;		------------------------------------   			;;
;;									;;
;;	Marc's key bindings for Linux.                                  ;;
;;									;;
;;**********************************************************************;;

;;(setq debug-on-error t)
(setq scroll-preserve-screen-position t)

;; Currently this is only fully defined for X-windows.

;; Define some helper functions for defining keys.

(defun rawkeyname (keysym)
  "return the keysym as a string without any S-, C-, etc prefix"
  (let ((name (symbol-name keysym)))
    (if (string-match "[SsCcAaMm]-" name)
	(substring name 2 (length name))
      name)))

(defun rawkeysym (keysym)
  "return the keysym without any S-, C-, etc prefix"
  (let ((name (symbol-name keysym)))
    (if (string-match "[SsCcAaMm]-" name)
	(make-symbol
	 (substring name 2 (length name)))
      keysym)))
	    
(defun rawkey (keysym)
  "return the key for keysym without any S-, C-, etc prefix"
  (vector (rawkeysym keysym)))

(defun S-key (keysym)
  "return the key for keysym with prefix of S-"
  (vector (make-symbol (concat "S-" (rawkeyname keysym)))))

(defun C-key (keysym)
  "return the key for keysym with prefix of C-"
  (vector (make-symbol (concat "C-" (rawkeyname keysym)))))


(defun define-keys (keysym keyfunc &optional goldfunc dblgoldfunc)
  "Define up to 3 functions for each function key.  This exists simply to
reduce the amount of code needed to set up all the keys."
  (if keyfunc
      (define-key global-map (rawkey keysym) keyfunc))
  (if goldfunc	
      (progn	
	(define-key global-map (S-key keysym) goldfunc)
	(define-key gold-map (rawkey keysym) goldfunc)))
  (if dblgoldfunc
      (progn
	(define-key global-map (C-key keysym) dblgoldfunc)
	(define-key dblgold-map (rawkey keysym) dblgoldfunc))))


;; Define prefix keymaps
(define-prefix-command 'gold-map)
(define-prefix-command 'dblgold-map)

;; F1 and KP-Divide are going to be the gold keys.
(define-key global-map  [f1]                	  'gold-map)
(define-key global-map  [S-f1]              	  'dblgold-map)
(define-key gold-map    [f1]                	  'dblgold-map)
(define-key global-map  [kp-divide]         	  'gold-map)
(define-key global-map  [S-kp-divide]       	  'dblgold-map)
(define-key gold-map    [kp-divide]         	  'dblgold-map)

;; The backspace key
(define-keys 'backspace 
  'backward-delete-char-untabify 'backward-kill-word 'backward-kill-word)

;; Top row of function keys.
(define-keys 'f2  'find-file                 	  'insert-file)
(define-keys 'f3  'select-buffer            	  'list-buffers)
(define-keys 'f4  'overwrite-mode            	  'auto-fill-mode)
(define-keys 'f5  'clipboard-kill-region     	  'copy-region-as-kill)
(define-keys 'f6  'other-window              	  'narrow-widen)
(define-keys 'f7  'split-window-vertically   	  'delete-other-windows 
                                             	  'split-window-horizontally)
(define-keys 'f8  'next-error        'compile
                                             	  'grep)
(define-keys 'f9  'save-buffer               	  'write-region-or-file)
(define-keys 'f10 'kill-buffer               	  'save-buffers-kill-emacs)
(define-keys 'f11 'power-search              	  'power-replace)
(define-keys 'f12 'repeat-complex-command    	  'shell)

;; Numeric function keypad
(define-keys 'kp-enter    'undo             	  'goto-line)
(define-keys 'kp-multiply 'eval-last-sexp   	  'ispell-buffer)
(define-keys 'kp-subtract 'kill-line        	  'kill-sentence   
                                            	  'kill-paragraph)
(define-keys 'kp-add      'kill-word              'what-line
                                                  'what-cursor-position)
(define-keys 'kp-decimal  'set-mark-command 	  'exchange-point-and-mark)
(define-keys 'kp-0  	  'fill-paragraph   	  'set-fill-prefix)
(define-keys 'kp-1        'power-search-toggle-direction 
                          'power-search-toggle-case-dependence)
(define-keys 'kp-2  	  'insert-register        'insert-register)
(define-keys 'kp-3  	  'copy-to-register       'copy-rectangle-to-register)
(define-keys 'kp-4  	  'power-replace-switch-mode 
                          'power-replace-display-state)
(define-keys 'kp-5  	  'yank-rectangle         'open-rectangle)
(define-keys 'kp-6  	  'kill-rectangle         'clear-rectangle)
(define-keys 'kp-7  	  'power-search-switch-mode 
                          'power-search-display-state)
(define-keys 'kp-8  	  'backward-word          'backward-sentence 
                                                  'backward-paragraph)
(define-keys 'kp-9  	  'forward-word           'forward-sentence 
                                                  'forward-paragraph)
;; Cursor Keys
;; Here the shifted and gold variants of the keys are not the same.

(defun mark-and-do ()
  "Activate region and then run the equivalent non-shifted key function."
  (interactive)
  (if (not mark-active)
      (push-mark nil nil t))
  (let ((key (aref (this-command-keys) 0)))
    (call-interactively 
     (key-binding (rawkey key)))))



;; Control-(and gold) Cursor
(define-key global-map  [C-right]           'forward-word)
(define-key gold-map    [right]             'forward-word)
(define-key dblgold-map [right]             'next-end-of-line)
(define-key global-map  [C-left]            'backward-word)
(define-key gold-map    [left]              'backward-word)
(define-key dblgold-map [left]              'prev-start-of-line)
(define-key gold-map    [up]                'beginning-of-buffer)
(define-key dblgold-map [up]                'beginning-of-buffer)
(define-key gold-map    [down]              'end-of-buffer)
(define-key dblgold-map [down]              'end-of-buffer)

;; Shift-Cursor
(define-key global-map  [S-right]           'mark-and-do)
(define-key global-map  [S-left]            'mark-and-do)
(define-key global-map  [S-up]	            'mark-and-do)		     
(define-key global-map  [S-down]            'mark-and-do)     
(define-key global-map  [S-end]	            'mark-and-do)		     
(define-key global-map  [S-prior]           'mark-and-do)		     
(define-key global-map  [S-next]            'mark-and-do)		     

;; The Tab key (there seem to be lots of definitions needed for this).
(define-keys 'tab         nil                     'hippie-expand
                                                  'hippie-expand)
(define-keys 'iso-lefttab nil                     'hippie-expand
                                                  'hippie-expand)
(define-key  gold-map     "\t"                    'hippie-expand)
(define-key  dblgold-map  "\t"                    'hippie-expand)

;; Redefine home, end, page-up, etc
(define-keys 'home        'prev-start-of-line     'prev-start-of-line
                                                  'beginning-of-buffer)
(define-keys 'end         'next-end-of-line       'next-end-of-line
                                                  'end-of-buffer)
;(define-keys 'prior       'scroll-down-in-place   'backward-paragraph
;                                                  'backward-paragraph)
;(define-keys 'next        'scroll-up-in-place     'forward-paragraph
;                                                  'forward-paragraph)
(define-keys 'insert      'yank                   'yank-pop
                                                  'overwrite-mode)
(define-keys 'delete      'delete-forward-char    'copy-region-as-kill
                                                  'narrow-widen)
;; Counters.  Very useful in macros.
(define-key gold-map    "="                 'user-counter-set)
(define-key gold-map    "+"                 'user-counter-insert)
(define-key esc-map     "="                 'user-counter-set)
(define-key esc-map     "+"                 'user-counter-insert)


;; Linux Console Keys
(define-key function-key-map "\e[1~" [home])
(define-key function-key-map "\e[4~" [end])
(define-key function-key-map "\e[2~" [insert])

;; And functions that really should be bound at all times.
(define-key esc-map "U"                      'upcase-word)
(define-key esc-map "u"                      'upcase-word)
(define-key esc-map "D"                      'downcase-word)
(define-key esc-map "d"                      'downcase-word)
(define-key esc-map "C"                      'capitalize-word)
(define-key esc-map "c"                      'capitalize-word)

;;==========================================================================
;; Load features required by keybindings defined above.
;;(require 'scroll-in-place)
(require 'odd-moves)                            ;; end of next line etc.
(require 'power-search)
;;(require 'extended-execute)

(autoload 'narrow-widen "narrow-toggle"
    "Toggle between narrowed and non-narrowed state.")
(autoload 'hippie-expand "hippie-exp"
  "Try to expand text before point, using multiple methods.")
(autoload 'user-counter-set "counter")
(autoload 'user-counter-increment "counter")
(autoload 'user-counter-insert "counter")
(autoload 'select-buffer "buf-sel")


;; That's all.
