(setq mouse-yank-at-point t)
(setq scroll-step 2)

;; define a macro to make ignoring errors 
;; easier

(defmacro ignore-errors (&rest forms)
  "short form for a condition case"
  (list 'condition-case 'nil 
	(cons 'progn forms)
	'(error nil)))

(setq default-major-mode 'text-mode)          
(setq default-fill-column 72)                 
(setq transient-mark-mode t)                  
(put 'eval-expression 'disabled nil)          

;; read the abbreviation file if it exists
(ignore-errors
  (quietly-read-abbrev-file))

; Set up hooks for other modes

;; Auto fill in Text mode
(add-hook 'text-mode-hook 
	  '(lambda () (turn-on-auto-fill)))

;; Turn on font-lock and line-number modes in a number of types of buffer.
(mapcar (lambda (hook)
	  (add-hook hook
	  '(lambda ()
	     (line-number-mode 1)
	     (if window-system 
		 (font-lock-mode 1)))))
	'(emacs-lisp-mode-hook lisp-mode-hook inferior-lisp-mode-hook
	  c++-mode-hook c-mode-hook tcl-mode-hook html-mode-hook
	  sgml-mode-hook))
 


;;
;; Modes to autoload
;; -----------------
;;


(autoload 'ispell-word "ispell"
  "Check the spelling of word in buffer." t)
(global-set-key "\e$" 'ispell-word)
(autoload 'ispell-region "ispell"
  "Check the spelling of region." t)
(autoload 'ispell-buffer "ispell"
  "Check the spelling of buffer." t)
(autoload 'ispell-complete-word "ispell"
  "Look up current word in dictionary and try to complete it." t)
(autoload 'ispell-change-dictionary "ispell"
  "Change ispell dictionary." t)

;;; Marc's bits
(autoload 'which "which" "Identify where emacs lisp files live." t)

;;
;; Lisp Functions
;; ==============
;;


(defun mouse-describe-function (event)
  "describe function under the mouse-cursor"
  (interactive "e")
  (save-excursion
    (mouse-set-point event)
    (let ((fn (function-called-at-point)))
      (describe-function fn)
      nil)))

(load "keys")
;;(load "marcs-misc")



