;; Various functions which respond according to whether the mark is active or
;; not.

(defadvice delete-forward-char (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, kill the region."
  (if (and transient-mark-mode mark-active)
      (kill-region (point) (mark))
    ad-do-it))

(defadvice backward-delete-char-untabify 
  (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, kill the region."
  (if (and transient-mark-mode mark-active)
      (kill-region (point) (mark))
    ad-do-it))

(defadvice set-mark-command (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, turn off the mark."
  (if (and transient-mark-mode mark-active)
      (deactivate-mark)
    ad-do-it))

(defadvice upcase-word (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, upcase the whole region."
  (if (and transient-mark-mode mark-active)
      (upcase-region (point) (mark))
    ad-do-it))

(defadvice downcase-word (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, upcase the whole region."
  (if (and transient-mark-mode mark-active)
      (downcase-region (point) (mark))
    ad-do-it))

(defadvice capitalize-word (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, upcase the whole region."
  (if (and transient-mark-mode mark-active)
      (capitalize-region (point) (mark))
    ad-do-it))

(defadvice fill-paragraph 
  (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, do fill-region-as-paragraph,
else do fill-paragraph."
  (if (and transient-mark-mode mark-active)
      (call-interactively 'fill-region-as-paragraph)
    ad-do-it))
      
(defadvice ispell-buffer (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, do ispell-region,
else do ispell-buffer."
  (if (and transient-mark-mode mark-active)
      (call-interactively 'ispell-region)
    ad-do-it))

(defadvice end-of-buffer (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, goto point-max,
else do end-of-buffer."
  (if (and transient-mark-mode mark-active)
      (goto-char (point-max))
    ad-do-it))

(defadvice beginning-of-buffer (around transient-mark-stuff first activate)
  "If in transient mark-mode and mark is active, goto point-min,
else do beginning-of-buffer."
  (if (and transient-mark-mode mark-active)
      (goto-char (point-min))
    ad-do-it))

(defun write-region-or-file ()
  "If in transient mark-mode and mark is active, write the region otherwise
write a file."
  (interactive)
  (if (and transient-mark-mode mark-active)
      (call-interactively 'write-region)
    (call-interactively 'write-file)))

;; General advice for messing about with indentation commands
(defun within-leading-whitespace-p ()
  "Predicate identifying whether point is within the current line's leading
whitespace or not."
  (let ((line (buffer-substring (save-excursion 
				  (beginning-of-line)
				  (point))
				(point))))
    (and (string-match "^\\s-*$" line) t)))
  
(defadvice indent-for-tab-command (around indent-stuff first activate)
  "Only do indentation if we are prior to the first non-space character,
otherwise do a real (normal) tab indentation."
    (if (within-leading-whitespace-p)
	ad-do-it
      (tab-to-tab-stop)))
	      
