;; marcs-shells.el
;;
;; A collection of hacks to make shell commands etc. a little more 
;; versatile.
;;

(autoload 'grep "px-grep"
  "A better version of emacs grep function.")

;; Now some cute stuff for emacs to handle passwords silently and execute
;; emacs commands on the basis of process output.

(add-hook 'shell-mode-hook 'add-shell-filter t)

(defun add-shell-filter ()
  "Set up the process filter for the shell process."
  (define-key shell-mode-map [backspace]
    'comint-delete-backward-char)
  (setq ring-prevent-duplicates t)
  (setq ring-despace-required t)
  (set-process-filter (get-buffer-process  (current-buffer))
		      'shell-filter))

(defvar shell-filter-echo nil
  "Variable to cause echoing of `##to-emacs##' messages.")

(defvar shell-password-prompt-regexp
  "^\\(old\\|new\\|enter\\)* *password:?[ \t]*$"
  "Regular expression that matches requests for passwords that should be 
entered without echo to the screen.")

(defun shell-filter (proc string)
  "Filter for shell to allow passwords to be read without echo, and to also
catch output aimed at emacs.
Such output begins ##to-emacs## and continues with an expression to evaluate.
Eg `##to-emacs## (find-file <filename-in-quotes>)'"
  (cond ((string-match shell-password-prompt-regexp string)
	 (let ((echo-keystrokes 0))
	   (comint-filter proc string)
	   (send-invisible nil)))
;;	   (comint-simple-send proc (read-password))))
	((string-match "^##to-emacs## " string)
	 (let* ((start (length "##to-emacs## "))
		(end (string-match "\n.*\\'" string))
		(newstr (substring string start end)))
	   (if shell-filter-echo
	       (message newstr)
	     (eval-expression (car (read-from-string newstr))))
	   (comint-filter proc (substring string (1+ end)))))
	(t
;; The following bit deals with output that contains ^M characters.
;; If they are followed by a newline, the ^M is removed, otherwise
;; the ^M is replaced by a newline.  This makes output from crappy
;; DOS or VMS programs much easier to view.
	 (let ((newstr ""))
	   (while (string-match "
" string)
	     (setq newstr 
		   (concat newstr 
			   (substring string 0 (match-beginning 0)) 
			   (cond ((eq (length string) (match-end 0))
				  "\n")
				 ((string= "\n" (substring string 
							   (match-end 0)
							   (1+ (match-end 0))))
				  "")
				 (t
				  "\n"))))
	     (setq string (substring string (match-end 0))))
	   (setq string (concat newstr string))
	   (comint-filter proc string)))))



(provide 'px-shells)

;; Some hacks to allow compatibility between emacs-19281 and earlier versions
;; Once we are no longer using earlier versions it should be possible to 
;; simply lose this crap.  It may be necessary to make some small changes to 
;; the stuff above at this time though.

(autoload 'comint-filter "patchitup.el")










