;; watched-undo.el
;; provide the means to watch undo and identify when changes are undone beyond
;; a watchpoint.

(require 'rollback)

(defvar buffer-undo-watchpoint nil)
(make-variable-buffer-local 'buffer-undo-watchpoint)

(defvar buffer-watchpoint-fn nil)
(make-variable-buffer-local 'buffer-watchpoint-fn)

(defun set-undo-watchpoint (fn)
  "Record FN as a function to be called when undo is being done past the 
current buffer state."
  (undo-boundary)
  (setq buffer-undo-watchpoint (first-undo-element buffer-undo-list)
	buffer-watchpoint-fn fn))

(defun reset-undo-watchpoint ()
  (setq buffer-undo-watchpoint nil
	buffer-watchpoint-fn nil))
        
(defadvice undo-more (before watched-undo activate)
  "If the saved watchpoint is reached, restore our original undo, from
prior to the watchpoint, call the watchpoint function and continue with
the undo operation."
  (if (eq buffer-undo-watchpoint (first-undo-element pending-undo-list))
      (progn
	(funcall buffer-watchpoint-fn)
	(reset-undo-watchpoint))))

(provide 'watched-undo)

