;;
;;  File: which.el
;; 
;;  Author: Mikael Djurfeldt (djurfeldt@nada.kth.se)
;;
;;  Last change: 22 July 1994
;;
;;  Copyright (C) Mikael Djurfeldt
;;
;;  ----------------------------------------------------------------------
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; either version 2, or (at your option)
;;  any later version.
;; 
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;; 
;;  You should have received a copy of the GNU General Public License
;;  along with your copy of Emacs; if not, write to the Free Software
;;  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;;  ----------------------------------------------------------------------
;;
;;  
;;  INSTALLATION
;;
;;  Put the following expression in your .emacs :
;;  
;;  (autoload 'which
;;            "which"
;;            "Give full pathname of the emacs library NAME"
;;            t)
;;
;;
;;  DESCRIPTION
;;
;;  The command which, which is inspired by the corresponding shell
;;  command, takes the name of an emacs library and gives the full
;;  pathname of the file emacs would load, given your load-path.
;;

(defun which (name)
  "Give full pathname of the emacs library NAME"
  (interactive "sLibrary name: ")
  (let ((path load-path)
	base ext pathname found)
    (while (and (not found) path)
      (setq base (concat (file-name-as-directory (or (car path) ".")) name)
	    path (cdr path)
	    ext '(".elc" ".el" ""))
      (while (and ext
		  (not (file-readable-p
			(setq pathname (concat base (car ext))))))
	(setq ext (cdr ext)))
      (setq found ext))
    (if (interactive-p)
	(message (if found pathname
		   (concat "no "
			   name
			   " in "
			   (mapconcat 'identity load-path " "))))
      (and found pathname))))

(provide 'which)
