;; This is an initial dumb implementation of wildkeys
;; At some point this should become a more widely applicable package and
;; the defadvice-based customisation for redo, etc should be moved into 
;; more appropriate packages.

(defvar advised-fns nil
  "List of advised functions for wild key purposes")

(defun is-advised (fn)
  "Is FN advised?"
  (member fn advised-fns))

(defun record-advised-fn (fn)
  "Record the fact that FN is advised"
  (setq advised-fns (cons fn advised-fns)))

(defmacro make-fn-wild (fn wild-fn purpose)
  "Advise FN to run WILD-FN.  The advice will have PURPOSE as its class"
  (unless (is-advised fn)
    (progn
      (record-advised-fn fn)
      `(defadvice ,fn (around ,purpose activate)
	 (or (call-interactively ',wild-fn)
	     ad-do-it)))))


(defun make-fnkey-wild (keys wild-fn purpose)
  "Advise the function associated with KEYS to run WILD-FN.  The advice will
have PURPOSE as its class"
  (let ((keyfn (key-binding keys)))
    (make-fn-wild keyfn wild-fn purpose)))



;; Define some base functions for use as wildkey functions.
(defun wild-tab-key (count)
  "Provide a basic function that may be advised using make-fnkey-wild."
  (interactive "p")
  (message "WILD")
  nil)

(defun wild-dot-key (count)
  "Provide a wild-key function for the '.' key.  This is to be bound
explicitly to the appropriate key in global-map."
  (interactive "p")
  (call-interactively 'self-insert-command))

(defun wild-minus-key (count)
  "Provide a wild-key function for the '-' key."
  (interactive "p")
  (call-interactively 'self-insert-command))

(defun wild-equal-key (count)
  "Provide a wild-key function for the '=' key."
  (interactive "p")
  (call-interactively 'self-insert-command))


;; Define explicit key-bindings for ".", "-", and "=" keys
(define-key global-map "-" 'wild-minus-key)
(define-key global-map "." 'wild-dot-key)
(define-key global-map "=" 'wild-equal-key)

(defadvice wild-minus-key (around redo activate)
  (if (member last-command '(redo-redo redo-pop redo-depop))
      (progn
	(message "redo-pop")
	(redo-pop))
    ad-do-it))

(defadvice wild-equal-key (around redo activate)
  (if (member last-command '(redo-redo redo-pop redo-depop))
      (progn
	(message "redo-depop")
	(redo-depop))
    ad-do-it))

(defadvice wild-dot-key (around redo activate)
  (if (member last-command '(redo-redo redo-pop))
      (redo-pop)
    ad-do-it))

(defadvice wild-minus-key (around yank activate)
  (if (member last-command '(yank yank-pop))
      (progn
	(yank-pop)
	(message "yank-pop"))
    ad-do-it))

(defadvice wild-equal-key (around yank activate)
  (if (member last-command '(yank yank-pop))
      (progn
	(yank-pop -1)
	(message "yank-depop"))
    ad-do-it))

(defadvice hippie-expand (before wild activate)
  ;; Make the function currently associated with the tab key, use wild-tab-key
  (make-fnkey-wild "\C-I" 'wild-tab-key 'hippie-expand))

