;;; fa-vm-hooks.el --- Customisations to allow vm and frame-assoc to 
;;;                    harmoniously coexist.
;; Copyright (C) 1996 Marc Munro

;; Author: Marc Munro (marc@bloodnok.demon.co.uk)
;; Version:  0.5
;; Keywords: frames, mail

;; This file is not part of GNU Emacs but is distributed according to the
;; terms of the GNU General Public License as described below.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;;; See frame-assoc.el for a description of what frame association is all
;;; about.  This file exists to integrate vm and frame-assoc together so
;;; that they are able to make best use of each others abilities.

;; Load the file that has been created by fa-adint.awk which will have
;; automatically generated most necessary advice for vm functions.

(load "fa-vm-mainadvice.el")

(defadvice vm (around vm-frame-assoc last)
  "vm is advised to name the Inbox frame VM."
  (let ((fa-defer-operations t))
    ad-do-it)
  (fa-perform-deferred)
  (fa-name-frame "VM"))

(defadvice vm-other-frame (around vm-frame-assoc last)
  "vm-other-frame is advised to name the Inbox frame VM."
  (let ((fa-defer-operations t))
    ad-do-it)
  (fa-perform-deferred)
  (fa-name-frame "VM"))

(defadvice vm-visit-folder-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame according to the folder name."
  (fa-name-frame "????"))

(defadvice vm-visit-virtual-folder-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame according to the folder name."
  (fa-name-frame "VM-????"))

(defadvice vm-mail-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Send"))

(defadvice vm-edit-message-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Edit"))

(defadvice vm-reply-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Reply"))

(defadvice vm-followup-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Followup"))

(defadvice vm-followup-include-text-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Followup"))

(defadvice vm-forward-message-all-headers-other-frame (after 
						       vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Forward"))

(defadvice vm-forward-message-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Forward"))

(defadvice vm-resend-message-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Resend"))

(defadvice vm-resend-bounced-message-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Resend"))

(defadvice vm-send-digest-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Send"))

(defadvice vm-send-rfc934-digest-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Send"))

(defadvice vm-send-rfc1153-digest-other-frame (after vm-frame-assoc last)
  "This function is advised to name the frame."
  (fa-name-frame "VM-Send"))



(defun vm-frame-assoc-activate ()
  "Hook function to activate this package."
  (ad-activate-regexp "vm-frame-assoc"))

(defun vm-frame-assoc-disable ()
  "Hook function to disable this package."
  (ad-deactivate-regexp "vm-frame-assoc"))

(add-hook 'fa-activate-hook 'vm-frame-assoc-activate)
(add-hook 'fa-deactivate-hook 'vm-frame-assoc-disable)




